<?php

namespace Drupal\search_api_daterange_filter\Plugin\views\Filter;

use Drupal\Component\Datetime\DateTimePlus;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\search_api\Item\FieldInterface;
use Drupal\search_api\Plugin\views\filter\SearchApiDate;
use Drupal\search_api\Plugin\views\query\SearchApiQuery;

/**
 * Daterange views filter.
 *
 * Extend SearchApiDate filter to include date range operations.
 *
 * @ViewsFilter("search_api_daterange")
 */
class SearchApiDaterange extends SearchApiDate {

  /**
   * {@inheritDoc}
   */
  public function operators() {
    $operators = parent::operators();
    $operators['includes'] = [
      'title' => $this->t('Includes'),
      'method' => 'opIncludes',
      'short' => $this->t('includes'),
      'values' => 1,
    ];

    return $operators;
  }

  /**
   * Filters by operator Includes.
   *
   * @param string $field
   *   The field name.
   */
  protected function opIncludes($field) {
    if (!$this->query instanceof SearchApiQuery) {
      $this->opSimple($field);
      return;
    }

    $end_field = $this->getDateEndField();
    if ($end_field === NULL) {
      $this->opSimple($field);
      return;
    }

    // Convert to ISO. UTC timezone is used since dates are stored in UTC.
    /** @var \Drupal\Component\Datetime\DateTimePlus $date */
    $date = new DateTimePlus($this->value['value'],
      new \DateTimeZone(DateTimeItemInterface::STORAGE_TIMEZONE));
    $end = $date->getTimestamp();
    // Needs the day 23:59:59 timestamp.
    $date->modify('+86399 sec');
    $start = $date->getTimestamp();

    // Prepare or conditions.
    $or = $this->query->createConditionGroup('OR');
    // Date is between start and end date.
    $and = $this->query->createConditionGroup('AND');
    $and->addCondition($this->realField, $start, '<=');
    $and->addCondition($end_field, $end, '>=');
    $or->addConditionGroup($and);

    // Start date is null and end date is greater than or equal to the date.
    $and = $this->query->createConditionGroup('AND');
    $and->addCondition($this->realField, NULL);
    $and->addCondition($end_field, $end, '>=');
    $or->addConditionGroup($and);

    // End date is null and start date is less than or equal to the date.
    $and = $this->query->createConditionGroup('AND');
    $and->addCondition($this->realField, $start, '<=');
    $and->addCondition($end_field, NULL);
    $or->addConditionGroup($and);

    // Add the condition group to the query.
    $this->query->addConditionGroup($or);
  }

  /**
   * Return the end field identifier.
   *
   * @return string|null
   *   The end field identifier.
   */
  protected function getDateEndField(): ?string {
    $end_field = NULL;
    if (!$this->query instanceof SearchApiQuery) {
      return $end_field;
    }
    /** @var \Drupal\search_api\Entity\Index $index */
    $index = $this->query->getIndex();
    /** @var \Drupal\search_api\Item\Field $original_field */
    $original_field = $index->getField($this->realField);
    if (!$original_field instanceof FieldInterface) {
      return $end_field;
    }

    // End date property path.
    $end_property_path = $original_field->getPropertyPath() . ':end_value';
    // Find the date end field.
    $filtered = array_filter($index->getFields(), function ($field) use ($end_property_path) {
      return $field->getPropertyPath() === $end_property_path;
    });
    if (!empty($filtered)) {
      // Set the first key as the end field.
      $end_field = array_keys($filtered)[0];
    }

    return $end_field;
  }

}
